#!/usr/bin/env python3
"""
Active Threat Hunter
Actively hunts for suspicious activities and threats on the system.
Parameters:
    - scan_type: Type of scan (quick, full, custom)
    - aggressive: Enable aggressive hunting mode
Requires: admin/root privileges
"""
import psutil
import platform
import subprocess
import logging
from datetime import datetime
import json
from pathlib import Path
import threading
import queue
import time
import re
import os
import socket
import hashlib

if platform.system() == 'Windows':
    import winreg
else:
    winreg = None

class ThreatHunter:
    def __init__(self, scan_type='quick', aggressive=False):
        self.scan_type = scan_type
        self.aggressive = aggressive
        self.is_windows = platform.system().lower() == 'windows'
        self.threats = queue.Queue()
        self.stop_event = threading.Event()
        self.setup_logging()
        
        # Known malicious patterns
        self.suspicious_patterns = {
            'processes': [
                r'cryptominer',
                r'winlogon\.exe$',
                r'svchost\.exe$',
                r'powershell.exe -enc'
            ],
            'connections': [
                r'\b(?:[0-9]{1,3}\.){3}[0-9]{1,3}\b:(?:443|80)',
                r'\.onion$',
                r'\.ru$'
            ],
            'files': [
                r'\.vbs$',
                r'\.bat$',
                r'\.ps1$',
                r'\.exe$'
            ]
        }
        
    def setup_logging(self):
        """Set up logging configuration"""
        log_dir = Path("threat_logs")
        log_dir.mkdir(exist_ok=True)
        
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s',
            handlers=[
                logging.FileHandler(log_dir / "threat_hunter.log"),
                logging.StreamHandler()
            ]
        )
        
    def calculate_file_hash(self, filepath):
        """Calculate SHA-256 hash of file"""
        try:
            sha256_hash = hashlib.sha256()
            with open(filepath, "rb") as f:
                for byte_block in iter(lambda: f.read(4096), b""):
                    sha256_hash.update(byte_block)
            return sha256_hash.hexdigest()
        except Exception:
            return None
            
    def scan_processes(self):
        """Scan running processes for suspicious activity"""
        logging.info("Scanning processes...")
        
        for proc in psutil.process_iter(['pid', 'name', 'cmdline', 'username']):
            try:
                # Skip if process has terminated
                if not proc.is_running():
                    continue
                    
                proc_info = proc.info
                suspicious = False
                reason = []
                
                # Check process name against patterns
                for pattern in self.suspicious_patterns['processes']:
                    if re.search(pattern, proc_info['name'], re.I):
                        suspicious = True
                        reason.append(f"Suspicious name pattern: {pattern}")
                        
                # Check command line
                if proc_info['cmdline']:
                    cmdline = ' '.join(proc_info['cmdline'])
                    if 'powershell' in cmdline.lower() and '-enc' in cmdline:
                        suspicious = True
                        reason.append("PowerShell encoded command detected")
                        
                # Check CPU usage if aggressive mode
                if self.aggressive:
                    try:
                        cpu_percent = proc.cpu_percent(interval=1.0)
                        if cpu_percent > 80:
                            suspicious = True
                            reason.append(f"High CPU usage: {cpu_percent}%")
                    except:
                        pass
                        
                if suspicious:
                    self.threats.put({
                        'type': 'suspicious_process',
                        'pid': proc_info['pid'],
                        'name': proc_info['name'],
                        'user': proc_info['username'],
                        'reason': reason,
                        'timestamp': datetime.now().isoformat()
                    })
                    
            except (psutil.NoSuchProcess, psutil.AccessDenied):
                continue
                
    def scan_connections(self):
        """Scan network connections for suspicious activity"""
        logging.info("Scanning network connections...")
        
        for conn in psutil.net_connections():
            try:
                if not conn.raddr:  # Skip if no remote address
                    continue
                    
                suspicious = False
                reason = []
                
                # Get process info
                try:
                    process = psutil.Process(conn.pid)
                    proc_name = process.name()
                except:
                    proc_name = "Unknown"
                    
                # Check remote address
                remote_addr = f"{conn.raddr.ip}:{conn.raddr.port}"
                
                # Check against patterns
                for pattern in self.suspicious_patterns['connections']:
                    if re.search(pattern, remote_addr, re.I):
                        suspicious = True
                        reason.append(f"Suspicious connection pattern: {pattern}")
                        
                # Check for high number of connections from same process
                if self.aggressive:
                    proc_connections = len([c for c in psutil.net_connections()
                                        if c.pid == conn.pid])
                    if proc_connections > 50:
                        suspicious = True
                        reason.append(f"High connection count: {proc_connections}")
                        
                if suspicious:
                    self.threats.put({
                        'type': 'suspicious_connection',
                        'pid': conn.pid,
                        'process': proc_name,
                        'local_addr': f"{conn.laddr.ip}:{conn.laddr.port}",
                        'remote_addr': remote_addr,
                        'status': conn.status,
                        'reason': reason,
                        'timestamp': datetime.now().isoformat()
                    })
                    
            except (psutil.NoSuchProcess, psutil.AccessDenied):
                continue
                
    def scan_startup(self):
        """Scan startup locations for suspicious entries"""
        logging.info("Scanning startup locations...")
        
        if self.is_windows:
            # Scan Windows registry startup locations
            startup_locations = [
                r"Software\Microsoft\Windows\CurrentVersion\Run",
                r"Software\Microsoft\Windows\CurrentVersion\RunOnce"
            ]
            
            for location in startup_locations:
                try:
                    key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, location)
                    i = 0
                    while True:
                        try:
                            name, value, _ = winreg.EnumValue(key, i)
                            # Check for suspicious entries
                            if any(re.search(pattern, value, re.I)
                                for pattern in self.suspicious_patterns['files']):
                                self.threats.put({
                                    'type': 'suspicious_startup',
                                    'location': location,
                                    'name': name,
                                    'value': value,
                                    'timestamp': datetime.now().isoformat()
                                })
                            i += 1
                        except WindowsError:
                            break
                except WindowsError:
                    continue
        else:
            # Scan Linux startup locations
            startup_locations = [
                '/etc/init.d',
                '/etc/rc.local',
                '/etc/crontab'
            ]
            
            for location in startup_locations:
                if os.path.exists(location):
                    try:
                        with open(location, 'r') as f:
                            content = f.read()
                            if any(re.search(pattern, content, re.I)
                                for pattern in self.suspicious_patterns['files']):
                                self.threats.put({
                                    'type': 'suspicious_startup',
                                    'location': location,
                                    'content': content[:200],  # First 200 chars
                                    'timestamp': datetime.now().isoformat()
                                })
                    except:
                        continue
                        
    def scan_files(self, directory):
        """Scan directory for suspicious files"""
        logging.info(f"Scanning directory: {directory}")
        
        try:
            for root, _, files in os.walk(directory):
                for file in files:
                    filepath = os.path.join(root, file)
                    
                    # Check file patterns
                    if any(re.search(pattern, file, re.I)
                        for pattern in self.suspicious_patterns['files']):
                        
                        # Calculate file hash if aggressive mode
                        file_hash = None
                        if self.aggressive:
                            file_hash = self.calculate_file_hash(filepath)
                            
                        self.threats.put({
                            'type': 'suspicious_file',
                            'path': filepath,
                            'name': file,
                            'hash': file_hash,
                            'timestamp': datetime.now().isoformat()
                        })
                        
        except Exception as e:
            logging.error(f"Error scanning directory {directory}: {str(e)}")
            
    def hunt_threats(self):
        """Start threat hunting"""
        logging.info(f"Starting threat hunt - Mode: {self.scan_type}")
        logging.info(f"Aggressive mode: {'Enabled' if self.aggressive else 'Disabled'}")
        
        start_time = time.time()
        
        # Start different scans based on type
        if self.scan_type in ['quick', 'full']:
            self.scan_processes()
            self.scan_connections()
            self.scan_startup()
            
        if self.scan_type == 'full':
            # Scan common directories
            directories = [
                os.path.expanduser('~'),
                os.environ.get('TEMP', '/tmp'),
                'C:\\Windows\\System32' if self.is_windows else '/bin'
            ]
            for directory in directories:
                if os.path.exists(directory):
                    self.scan_files(directory)
                    
        # Process and save results
        threats_list = []
        while not self.threats.empty():
            threats_list.append(self.threats.get())
            
        self.save_results(threats_list, time.time() - start_time)
        
    def save_results(self, threats, duration):
        """Save threat hunting results"""
        results = {
            'timestamp': datetime.now().isoformat(),
            'scan_type': self.scan_type,
            'aggressive_mode': self.aggressive,
            'duration': duration,
            'total_threats': len(threats),
            'threats': threats
        }
        
        # Save to file
        output_dir = Path("threat_logs")
        output_file = output_dir / f"threats_{datetime.now().strftime('%Y%m%d_%H%M%S')}.json"
        with open(output_file, 'w') as f:
            json.dump(results, f, indent=2)
            
        logging.info(f"Results saved to {output_file}")
        
        # Print summary
        print("\nThreat Hunting Summary")
        print("=" * 50)
        print(f"Scan Type: {self.scan_type}")
        print(f"Duration: {duration:.1f} seconds")
        print(f"Total Threats Found: {len(threats)}")
        
        if threats:
            print("\nDetected Threats:")
            for threat in threats:
                print(f"\n{threat['type']}:")
                if 'name' in threat:
                    print(f"Name: {threat['name']}")
                if 'path' in threat:
                    print(f"Path: {threat['path']}")
                if 'reason' in threat:
                    print(f"Reason: {', '.join(threat['reason'])}")

if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(description="Active Threat Hunter")
    parser.add_argument("--type", choices=['quick', 'full', 'custom'],
                      default='quick', help="Type of scan to perform")
    parser.add_argument("--aggressive", action="store_true",
                      help="Enable aggressive hunting mode")
    args = parser.parse_args()
    
    hunter = ThreatHunter(args.type, args.aggressive)
    hunter.hunt_threats()
